internal

package
v0.2.2 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Apr 7, 2023 License: Apache-2.0 Imports: 7 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Validate

func Validate(data any) error

Validate takes in a reference to a data struct (pointer, map of, slice of) and validates each of its fields against their dv8 field tags. It recurses into nested structs.

Types

type Validator added in v0.1.4

type Validator interface {
	Validate() error
}

Validator implements a single method that returns an error if a struct is invalid. DV8 calls this function during validation on types that implements it.

Jump to